@@ -16,8 +16,8 @@ running Editra.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: Editra.py 67100 2011-03-01 15:12:16Z CJP $"
-__revision__ = "$Revision: 67100 $"
+__svnid__ = "$Id: Editra.py 69061 2011-09-11 17:04:41Z CJP $"
+__revision__ = "$Revision: 69061 $"
 
 #--------------------------------------------------------------------------#
 # Dependencies
@@ -66,6 +66,7 @@ import ed_event
 import updater
 import plugin
 import ed_ipc
+import ed_session
 import ebmlib
 from syntax import synglob
 
@@ -125,12 +126,20 @@ class Editra(wx.App, events.AppEventHandlerMixin):
                             p = ebmlib.GetAbsPath(p)
                         except:
                             pass
-                        nargs.append(p)
-                    exml.SetFiles(nargs)
-                exml.SetArgs(opts.items())
+                        fxml = ed_ipc.IPCFile()
+                        fxml.value = p
+                        nargs.append(fxml)
+                    exml.filelist = nargs
+                arglist = list()
+                for arg, val in opts.items():
+                    axml = ed_ipc.IPCArg()
+                    axml.name = arg
+                    axml.value = val
+                    arglist.append(axml)
+                exml.arglist = arglist
 
                 # TODO: need to process other command line options as well i.e) -g
-                self._log("[app][info] Sending: %s" % exml.GetXml())
+                self._log("[app][info] Sending: %s" % exml.Xml)
                 rval = ed_ipc.SendCommands(exml, profiler.Profile_Get('SESSION_KEY'))
                 # If sending the command failed then let the editor startup
                 # a new instance
@@ -272,7 +281,7 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         self._pluginmgr.WritePluginConfig()
         profiler.TheProfile.Write(profiler.Profile_Get('MYPROFILE'))
         if not self._lock or force:
-            if hasattr(self, '_server'):
+            if getattr(self, '_server', None):
                 self._server.Shutdown()
 
             try:
@@ -547,23 +556,24 @@ class Editra(wx.App, events.AppEventHandlerMixin):
         self._log("[app][info] IN OnCommandReceived")
         cmds = evt.GetCommands()
         if isinstance(cmds, ed_ipc.IPCCommand):
-            self._log("[app][info] OnCommandReceived %s" % cmds.GetXml())
-            if not len(cmds.GetFiles()):
+            self._log("[app][info] OnCommandReceived %s" % cmds.Xml)
+            if not len(cmds.filelist):
                 self.OpenNewWindow()
             else:
                 # TODO: change goto line handling to require one
                 #       arg per file specified on the command line
                 #       i.e) -g 23,44,100
                 line = -1
-                for arg, val in cmds.GetArgs():
+                for argobj in cmds.arglist:
+                    arg = argobj.name
                     if arg == '-g':
-                        line = val
+                        line = int(argobj.value)
                         if line > 0:
                             line -= 1
                         break
 
-                for fname in cmds.GetFiles():
-                    self.OpenFile(fname, line)
+                for fname in cmds.filelist:
+                    self.OpenFile(fname.value, line)
 
     def OnCloseWindow(self, evt):
         """Close the currently active window
@@ -757,13 +767,19 @@ def InitConfig():
         else:
             dev_tool.DEBUGP("[InitConfig][info] Updating Profile to current version")
 
+            # When upgrading from an older version make sure all
+            # config directories are available.
+            for cfg in ("cache", "styles", "plugins", "profiles", "sessions"):
+                if not util.HasConfigDir(cfg):
+                    util.MakeConfigDir(cfg)
+
             # Load and update profile
             pstr = profiler.GetProfileStr()
             pstr = util.RepairConfigState(pstr)
             profiler.TheProfile.Load(pstr)
             profiler.TheProfile.Update()
 
-            #---- Temporary Profile Adaptions ----#
+            #---- Temporary Profile Adaption ----#
 
             # Added after 0.5.32
             mconfig = profiler.Profile_Get('LEXERMENU', default=None)
@@ -796,21 +812,22 @@ def InitConfig():
 
             # After 0.4.65 LAST_SESSION now points a session file and not
             # to a list of files to open.
+            ed_glob.CONFIG['SESSION_DIR'] = util.ResolvConfigDir(u"sessions")
+            smgr = ed_session.EdSessionMgr()
             sess = profiler.Profile_Get('LAST_SESSION')
             if isinstance(sess, list):
-                profiler.Profile_Set('LAST_SESSION', u'')
+                profiler.Profile_Set('LAST_SESSION', smgr.DefaultSession)
+            else:
+                # After 0.6.58 session is reduced to a name instead of path
+                if os.path.sep in sess:
+                    name = smgr.SessionNameFromPath(sess)
+                    profiler.Profile_Set('LAST_SESSION', name)
 
-            #---- End Temporary Profile Adaptions ----#
+            #---- End Temporary Profile Adaption ----#
 
             # Write out updated profile
             profiler.TheProfile.Write(pstr)
 
-            # When upgrading from an older version make sure all
-            # config directories are available.
-            for cfg in ("cache", "styles", "plugins", "profiles", "sessions"):
-                if not util.HasConfigDir(cfg):
-                    util.MakeConfigDir(cfg)
-
             profile_updated = True
     else:
         # Fresh install
