@@ -1,6 +1,6 @@
 ###############################################################################
 # Name: __init__.py                                                           #
-# Purpose: Editra Buisness Model Library                                      #
+# Purpose: Editra Business Model Library                                      #
 # Author: Cody Precord <cprecord@editra.org>                                  #
 # Copyright: (c) 2009 Cody Precord <staff@editra.org>                         #
 # Licence: wxWindows Licence                                                  #
@@ -12,8 +12,8 @@ Editra Business Model Library:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: __init__.py 66855 2011-02-06 20:03:25Z CJP $"
-__revision__ = "$Revision: 66855 $"
+__cvsid__ = "$Id: __init__.py 69022 2011-09-07 22:03:06Z CJP $"
+__revision__ = "$Revision: 69022 $"
 
 #-----------------------------------------------------------------------------#
 
@@ -37,7 +37,9 @@ from e_weblib import *
 
 # Misc
 from miscutil import *
+from _efactory import *
 from cmenumgr import *
 from efilehist import *
 from osutil import *
-
+from _threads import *
+from _trash import *
