@@ -13,19 +13,44 @@ Editra.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: $"
-__revision__ = "$Revision:  $"
+__svnid__ = "$Id: ed_basewin.py 69061 2011-09-11 17:04:41Z CJP $"
+__revision__ = "$Revision: 69061 $"
 
 #--------------------------------------------------------------------------#
 # Imports
 import wx
 
 # Local Imports
+import ed_msg
 import eclib
 import util
 
 #--------------------------------------------------------------------------#
 
+def FindMainWindow(window):
+        """Find the MainWindow of the given window
+        @return: MainWindow or None
+
+        """
+        def IsMainWin(win):
+            """Check if the given window is a main window"""
+            return getattr(win, '__name__', '') == 'MainWindow'
+
+        if IsMainWin(window):
+            return window
+        # else start looking up the parent hierarchy
+        tlw = window.GetTopLevelParent()
+        if IsMainWin(tlw):
+            return tlw
+        elif hasattr(tlw, 'GetParent'):
+            tlw = tlw.GetParent()
+            if IsMainWin(tlw):
+                return tlw
+
+        return None
+
+#--------------------------------------------------------------------------#
+
 class EdBaseDialog(eclib.ECBaseDlg):
     """Editra Dialog Base Class"""
     def __init__(self, parent, id=wx.ID_ANY, title=u"",
@@ -65,6 +90,25 @@ class EdBaseCtrlBox(eclib.ControlBox):
     def __init__(self, parent):
         super(EdBaseCtrlBox, self).__init__(parent)
 
+        ed_msg.Subscribe(self._OnFontChange, ed_msg.EDMSG_DSP_FONT)
+        self.Bind(wx.EVT_WINDOW_DESTROY, self._OnDestroy)
+
+    def _OnDestroy(self, evt):
+        if self and evt.GetEventObject is self:
+            ed_msg.Unsubscribe(self._OnFontChange)
+
+    def _OnFontChange(self, msg):
+        """Update font of all controls"""
+        if not self:
+            return
+        font = msg.GetData()
+        if isinstance(font, wx.Font):
+            for pos in (wx.TOP, wx.BOTTOM, wx.LEFT, wx.RIGHT):
+                cbar = self.GetControlBar(pos)
+                if cbar:
+                    for child in cbar.GetChildren():
+                        child.SetFont(font)
+
     def AddPlateButton(self, lbl=u"", bmp=-1,
                        align=wx.ALIGN_LEFT, cbarpos=wx.TOP):
         """Add an eclib.PlateButton to the ControlBar specified by
@@ -78,15 +122,7 @@ class EdBaseCtrlBox(eclib.ControlBox):
         """
         ctrlbar = self.GetControlBar(cbarpos)
         assert ctrlbar is not None, "No ControlBar at cbarpos"
-        if not isinstance(bmp, wx.Bitmap):
-            assert isinstance(bmp, int)
-            bmp = wx.ArtProvider.GetBitmap(str(bmp), wx.ART_MENU)
-        if bmp.IsNull() or not bmp.IsOk():
-            bmp = None
-        btn = eclib.PlateButton(ctrlbar, wx.ID_ANY, lbl, bmp,
-                                style=eclib.PB_STYLE_NOBG)
-        ctrlbar.AddControl(btn, align)
-        return btn
+        return ctrlbar.AddPlateButton(lbl, bmp, align)
 
     def CreateControlBar(self, pos=wx.TOP):
         """Override for CreateControlBar to automatically set the
@@ -94,6 +130,26 @@ class EdBaseCtrlBox(eclib.ControlBox):
 
         """
         cbar = super(EdBaseCtrlBox, self).CreateControlBar(pos)
+        cbar.__class__ = EdBaseCtrlBar
         if wx.Platform == '__WXGTK__':
             cbar.SetWindowStyle(eclib.CTRLBAR_STYLE_DEFAULT)
         return cbar
+
+class EdBaseCtrlBar(eclib.ControlBar):
+    def AddPlateButton(self, lbl=u"", bmp=-1, align=wx.ALIGN_LEFT):
+        """Add an eclib.PlateButton 
+        @keyword lbl: Button Label
+        @keyword bmp: Bitmap or EditraArtProvider ID
+        @keyword align: button alignment
+        @return: PlateButton instance
+
+        """
+        if not isinstance(bmp, wx.Bitmap):
+            assert isinstance(bmp, int)
+            bmp = wx.ArtProvider.GetBitmap(str(bmp), wx.ART_MENU)
+        if bmp.IsNull() or not bmp.IsOk():
+            bmp = None
+        btn = eclib.PlateButton(self, wx.ID_ANY, lbl, bmp,
+                                style=eclib.PB_STYLE_NOBG)
+        self.AddControl(btn, align)
+        return btn
