@@ -14,8 +14,8 @@ Text editor buffer view control for the main notebook
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_editv.py 67542 2011-04-19 00:26:29Z CJP $"
-__revision__ = "$Revision: 67542 $"
+__svnid__ = "$Id: ed_editv.py 69268 2011-10-01 19:50:54Z CJP $"
+__revision__ = "$Revision: 69268 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -66,6 +66,8 @@ def modalcheck(func):
 class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
     """Tab editor view for main notebook control."""
     ID_NO_SUGGEST = wx.NewId()
+    ID_CLOSE_TAB = wx.NewId()
+    ID_CLOSE_ALL_TABS = wx.NewId()
     DOCMGR = DocPositionMgr()
 
     def __init__(self, parent, id_=wx.ID_ANY, pos=wx.DefaultPosition,
@@ -111,17 +113,19 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
         self.Bind(wx.EVT_LEFT_UP, self.OnSetFocus)
         self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy, self)
 
-        ed_msg.Subscribe(self.OnConfigMsg,
-                         ed_msg.EDMSG_PROFILE_CHANGE + ('SPELLCHECK',))
-        ed_msg.Subscribe(self.OnConfigMsg,
-                         ed_msg.EDMSG_PROFILE_CHANGE + ('AUTOBACKUP',))
-        ed_msg.Subscribe(self.OnConfigMsg,
-                         ed_msg.EDMSG_PROFILE_CHANGE + ('SYNTHEME',))
-        ed_msg.Subscribe(self.OnConfigMsg,
-                         ed_msg.EDMSG_PROFILE_CHANGE + ('SYNTAX',))
+        # Subscribe for configuration updates
+        for opt in ('AUTOBACKUP', 'SYNTHEME', 'SYNTAX', 'BRACKETHL', 'GUIDES',
+                    'SHOW_EDGE', 'EDGE', 'CODE_FOLD', 'AUTO_COMP',
+                    'AUTO_INDENT', 'HLCARETLINE', 'SPELLCHECK', 'VI_EMU',
+                    'VI_NORMAL_DEFAULT', 'USETABS', 'TABWIDTH', 'INDENTWIDTH',
+                    'BSUNINDENT', 'EOL_MODE', 'AALIASING', 'SHOW_EOL', 'SHOW_LN',
+                    'SHOW_WS', 'WRAP', 'VIEWVERTSPACE'):
+            ed_msg.Subscribe(self.OnConfigMsg,
+                             ed_msg.EDMSG_PROFILE_CHANGE + (opt,))
 
     def OnDestroy(self, evt):
-        if evt.GetId() == self.GetId():
+        """Cleanup message handlers on destroy"""
+        if evt.Id == self.Id:
             ed_msg.Unsubscribe(self.OnConfigMsg)
         evt.Skip()
 
@@ -259,9 +263,9 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
         menu.Append(ed_glob.ID_MOVE_TAB, _("Move Tab to New Window"))
         menu.AppendSeparator()
         menu.Append(ed_glob.ID_SAVE, _("Save \"%s\"") % ptxt)
-        menu.Append(ed_glob.ID_CLOSE, _("Close \"%s\"") % ptxt)
+        menu.Append(EdEditorView.ID_CLOSE_TAB, _("Close \"%s\"") % ptxt)
         menu.Append(ed_glob.ID_CLOSE_OTHERS, _("Close Other Tabs"))
-        menu.Append(ed_glob.ID_CLOSEALL, _("Close All"))
+        menu.Append(EdEditorView.ID_CLOSE_ALL_TABS, _("Close All"))
         menu.AppendSeparator()
         menu.Append(ed_glob.ID_COPY_FILE, _("Copy Filename"))
         menu.Append(ed_glob.ID_COPY_PATH, _("Copy Full Path"))
@@ -350,10 +354,11 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
             parent = self.GetParent()
             if hasattr(parent, 'CloseOtherPages'):
                 parent.CloseOtherPages()
-        elif wx.Platform == '__WXGTK__' and \
-             e_id in (ed_glob.ID_CLOSE, ed_glob.ID_CLOSEALL):
+        elif e_id in (EdEditorView.ID_CLOSE_TAB, EdEditorView.ID_CLOSE_ALL_TABS):
             # Need to relay events up to toplevel window on GTK for them to
-            # be processed. On other platforms the propagate by them selves.
+            # be processed. On other platforms the propagate by themselves.
+            evt.SetId({ EdEditorView.ID_CLOSE_TAB : ed_glob.ID_CLOSE,
+                        EdEditorView.ID_CLOSE_ALL_TABS : ed_glob.ID_CLOSEALL}.get(e_id))
             wx.PostEvent(self.GetTopLevelParent(), evt)
         else:
             evt.Skip()
@@ -377,14 +382,45 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
             self._spell.setDefaultLanguage(mdata.get('dict', 'en_US'))
             if not self._spell_data['enabled']:
                 self._spell.clearAll()
-        elif mtype == 'AUTOBACKUP':
-            self.EnableAutoBackup(Profile_Get('AUTOBACKUP'))
-        elif mtype == 'SYNTHEME':
-            self.UpdateAllStyles()
-        elif mtype == 'SYNTAX':
-            self.SyntaxOnOff(Profile_Get('SYNTAX'))
+            return
         elif mtype == 'AUTO_COMP_EX':
             self.ConfigureAutoComp()
+            return
+        elif mtype in ('VI_EMU', 'VI_NORMAL_DEFAULT'):
+            self.SetViEmulationMode(Profile_Get('VI_EMU'),
+                                    Profile_Get('VI_NORMAL_DEFAULT'))
+            return
+        elif mtype == 'VIEWVERTSPACE':
+            self.SetEndAtLastLine(not Profile_Get('VIEWVERTSPACE'))
+
+        # Update other settings
+        cfgmap = { 'AUTOBACKUP' : self.EnableAutoBackup,
+                   'SYNTHEME'   : self.UpdateAllStyles,
+                   'SYNTAX'     : self.SyntaxOnOff,
+                   'BRACKETHL'  : self.ToggleBracketHL,
+                   'GUIDES'     : self.SetIndentationGuides,
+                   'SHOW_EDGE'  : self.SetViewEdgeGuide,
+                   'EDGE'       : self.SetViewEdgeGuide,
+                   'CODE_FOLD'  : self.FoldingOnOff,
+                   'AUTO_COMP'  : self.SetAutoComplete,
+                   'AUTO_INDENT': self.ToggleAutoIndent,
+                   'HLCARETLINE': self.SetCaretLineVisible,
+                   'USETABS'    : self.SetUseTabs,
+                   'BSUNINDENT' : self.SetBackSpaceUnIndents,
+                   'EOL_MODE'   : self.SetEOLMode,
+                   'AALIASING'  : self.SetUseAntiAliasing,
+                   'SHOW_EOL'   : self.SetViewEOL,
+                   'SHOW_LN'    : self.ToggleLineNumbers,
+                   'SHOW_WS'    : self.SetViewWhiteSpace,
+                   'WRAP'       : self.SetWrapMode                   }
+        if mtype in cfgmap:
+            cfgmap[mtype](Profile_Get(mtype))
+            return
+
+        cfgmap2 = { 'TABWIDTH'   : self.SetTabWidth,
+                    'INDENTWIDTH': self.SetIndent }
+        if mtype in cfgmap2:
+            cfgmap2[mtype](Profile_Get(mtype, 'int'))
 
     def OnContextMenu(self, evt):
         """Handle right click menu events in the buffer"""
