@@ -15,8 +15,8 @@ text documents and files.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_search.py 67193 2011-03-14 18:40:21Z CJP $"
-__revision__ = "$Revision: 67193 $"
+__svnid__ = "$Id: ed_search.py 69067 2011-09-11 23:28:42Z CJP $"
+__revision__ = "$Revision: 69067 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -34,6 +34,8 @@ import iface
 from profiler import Profile_Get, Profile_Set
 import eclib
 import ebmlib
+import ed_basewin
+import ed_thread
 
 #--------------------------------------------------------------------------#
 # Globals
@@ -47,7 +49,7 @@ class EdSearchEngine(ebmlib.SearchEngine):
                  matchcase=True, wholeword=False):
         super(EdSearchEngine, self).__init__(query, regex, down,
                                              matchcase, wholeword)
-        # Atttributes
+        # Attributes
         self._offset = 0
 
     def FormatResult(self, fname, lnum, match):
@@ -101,7 +103,7 @@ class SearchController(object):
         self._parent   = owner
         self._stc      = getstc
         self._finddlg  = None
-        self._posinfo  = dict(scroll=0, start=0, found=0, ldir=None)
+        self._posinfo  = dict(scroll=0, start=0, found=-1, ldir=None)
         self._data     = self._InitFindData()
         self._li_choices = list()
         self._li_sel   = 0
@@ -348,7 +350,7 @@ class SearchController(object):
                       _("Find Count"),
                       wx.ICON_INFORMATION|wx.OK)
 
-    def OnFind(self, evt, findnext=False):
+    def OnFind(self, evt, findnext=False, incremental=False):
         """Do an incremental search in the currently buffer
         @param evt: EVT_FIND, EVT_FIND_NEXT
         @keyword findnext: force a find next action
@@ -394,24 +396,24 @@ class SearchController(object):
         #      that are large.
         self._engine.SetSearchPool(stc.GetTextRaw())
 
-        # Check if the direction changed
-        ldir = self._posinfo['ldir']
-        if isdown:
-            self._posinfo['ldir'] = 'down'
-        else:
-            self._posinfo['ldir'] = 'up'
-
         # Get the search start position
         if evt.GetEventType() == eclib.edEVT_FIND:
-            spos = self._posinfo['found']
+            if not incremental:
+                spos = stc.CurrentPos
+            else:
+                # For incremental search redo search starting from last found
+                # position.
+                spos = self._posinfo['found']
+                if spos < 0:
+                    spos = stc.CurrentPos
         else:
-            spos = stc.GetCurrentPos()
-            if ldir != self._posinfo['ldir']:
-                start, end = stc.GetSelection()
-                if ldir == 'down':
-                    spos = start
+            spos = stc.CurrentPos
+            start, end = stc.GetSelection()
+            if start != end:
+                if isdown:
+                    spos = max(start, end)
                 else:
-                    spos = end
+                    spos = min(start, end)
 
         # Do the find
         match = self._engine.Find(spos)
@@ -423,7 +425,7 @@ class SearchController(object):
                 end = end + spos
                 stc.SetSelection(start, end)
             else:
-                stc.SetSelection(end, start)
+                stc.SetSelection(start, end)
 
             # Ensure caret and the line its in is exposed
             stc.EnsureCaretVisible()
@@ -451,14 +453,11 @@ class SearchController(object):
 
                 self._posinfo['found'] = start
 
-                match = list(match)
-                if not isdown:
-                    match.reverse()
-                stc.SetSelection(match[0], match[1])
+                stc.SetSelection(start, end)
                 
                 # Ensure caret and the line its in is exposed
                 stc.EnsureCaretVisible()
-                line = stc.LineFromPosition(match[0])
+                line = stc.LineFromPosition(start)
                 stc.EnsureVisible(line)
             else:
                 self._posinfo['found'] = -1
@@ -557,8 +556,11 @@ class SearchController(object):
             Profile_Set('SEARCH_LOC', choices)
             Profile_Set('SEARCH_FILTER', self._filters)
             self._StoreFindData()
-        evt.Skip()
-        self._parent.SetFocus()
+            self._finddlg.Destroy()
+            self._finddlg = None
+        buff = wx.GetApp().GetCurrentBuffer()
+        if buff:
+            buff.SetFocus()
 
     def OnOptionChanged(self, evt):
         """Handle when the find options are changed in the dialog"""
@@ -852,10 +854,12 @@ class EdSearchCtrl(wx.SearchCtrl):
             for child in self.GetChildren():
                 if isinstance(child, wx.TextCtrl):
                     child.Bind(wx.EVT_KEY_UP, self.ProcessEvent)
+                    child.Bind(wx.EVT_KEY_DOWN, self.ProcessEvent)
                     self._txtctrl = child
                     break
         else:
             self.Bind(wx.EVT_KEY_UP, self.ProcessEvent)
+            self.Bind(wx.EVT_KEY_DOWN, self.ProcessEvent)
         self.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.OnCancel)
         self.Bind(wx.EVT_MENU, self.OnHistMenu)
 
@@ -886,9 +890,10 @@ class EdSearchCtrl(wx.SearchCtrl):
         evt.SetFindString(self.GetValue())
         self.FindService.OnFindAll(evt)
 
-    def DoSearch(self, next=True):
+    def DoSearch(self, next=True, incremental=False):
         """Do the search and move the selection
         @keyword next: search next or previous
+        @keyword incremental: is this an incremental search
 
         """
         s_cmd = eclib.edEVT_FIND
@@ -904,7 +909,7 @@ class EdSearchCtrl(wx.SearchCtrl):
         evt = eclib.FindEvent(s_cmd, flags=self._flags)
         self._last = self.GetValue()
         evt.SetFindString(self.GetValue())
-        self.FindService.OnFind(evt)
+        self.FindService.OnFind(evt, incremental=incremental)
 
         # Give feedback on whether text was found or not
         if self.FindService.GetLastFound() < 0 and len(self.GetValue()) > 0:
@@ -1066,11 +1071,19 @@ class EdSearchCtrl(wx.SearchCtrl):
         @param evt: the event that called this handler
 
         """
+        e_key = evt.GetKeyCode()
         if evt.GetEventType() != wx.wxEVT_KEY_UP:
-            evt.Skip()
+            if e_key in (wx.WXK_UP, wx.WXK_DOWN):
+                buff = wx.GetApp().GetCurrentBuffer()
+                if isinstance(buff, wx.stc.StyledTextCtrl):
+                    val = -1
+                    if e_key == wx.WXK_DOWN:
+                        val = 1
+                    buff.ScrollLines(val)
+            else:
+                evt.Skip()
             return
 
-        e_key = evt.GetKeyCode()
         if e_key == wx.WXK_ESCAPE:
             # TODO change to more safely determine the context
             # Currently control is only used in command bar
@@ -1107,7 +1120,7 @@ class EdSearchCtrl(wx.SearchCtrl):
             # Don't do incremental searches when the RegEx flag is set in order
             # to avoid errors in compiling the expression
             if not self.IsRegEx():
-                self.DoSearch(next=True)
+                self.DoSearch(next=True, incremental=True)
 
     def OnCancel(self, evt):
         """Cancels the Search Query
@@ -1242,7 +1255,7 @@ class EdFindResults(plugin.Plugin):
 
 #-----------------------------------------------------------------------------#
 
-class SearchResultScreen(eclib.ControlBox):
+class SearchResultScreen(ed_basewin.EdBaseCtrlBox):
     """Screen for displaying search results and navigating to them"""
     def __init__(self, parent):
         """Create the result screen
@@ -1265,9 +1278,9 @@ class SearchResultScreen(eclib.ControlBox):
         # Event Handlers
         self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy, self)
         self.Bind(wx.EVT_BUTTON,
-                  lambda evt: self._list.Clear(), id=wx.ID_CLEAR)
+                  lambda evt: self._list.Clear(), self._clearb)
         self.Bind(wx.EVT_BUTTON,
-                  lambda evt: self.CancelSearch(), id=wx.ID_CANCEL)
+                  lambda evt: self.CancelSearch(), self._cancelb)
         self._list.Bind(eclib.EVT_TASK_START, self.OnTaskStart)
         self._list.Bind(eclib.EVT_TASK_COMPLETE, self.OnTaskComplete)
 
@@ -1281,33 +1294,17 @@ class SearchResultScreen(eclib.ControlBox):
 
     def __DoLayout(self):
         """Layout and setup the results screen ui"""
-        ctrlbar = eclib.ControlBar(self, style=eclib.CTRLBAR_STYLE_GRADIENT)
-        if wx.Platform == '__WXGTK__':
-            ctrlbar.SetWindowStyle(eclib.CTRLBAR_STYLE_DEFAULT)
-
+        ctrlbar = self.CreateControlBar(wx.TOP)
         ctrlbar.AddStretchSpacer()
 
         # Cancel Button
-        cbmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_STOP), wx.ART_MENU)
-        if cbmp.IsNull() or not cbmp.IsOk():
-            cbmp = wx.ArtProvider.GetBitmap(wx.ART_ERROR,
-                                            wx.ART_MENU, (16, 16))
-        cancel = eclib.PlateButton(ctrlbar, wx.ID_CANCEL, _("Cancel"),
-                                      cbmp, style=eclib.PB_STYLE_NOBG)
+        cancel = self.AddPlateButton(_("Cancel"), ed_glob.ID_STOP, wx.ALIGN_RIGHT)
         self._cancelb = cancel
-        ctrlbar.AddControl(cancel, wx.ALIGN_RIGHT)
 
         # Clear Button
-        cbmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_DELETE), wx.ART_MENU)
-        if cbmp.IsNull() or not cbmp.IsOk():
-            cbmp = None
-        clear = eclib.PlateButton(ctrlbar, wx.ID_CLEAR, _("Clear"),
-                                     cbmp, style=eclib.PB_STYLE_NOBG)
+        clear = self.AddPlateButton(_("Clear"), ed_glob.ID_DELETE, wx.ALIGN_RIGHT)
         self._clearb = clear
-        ctrlbar.AddControl(clear, wx.ALIGN_RIGHT)
 
-        ctrlbar.SetVMargin(1, 1)
-        self.SetControlBar(ctrlbar)
         self.SetWindow(self._list)
 
     def GetDisplayedLines(self):
@@ -1340,8 +1337,8 @@ class SearchResultScreen(eclib.ControlBox):
         ed_msg.PostMessage(ed_msg.EDMSG_UI_SB_TXT,
                            (ed_glob.SB_INFO, _("Search complete")))
 
-        # Let the update buffer be flushed
-        wx.YieldIfNeeded()
+        # Flush any remaining text to the output buffer
+        self._list.FlushBuffer()
  
         # Add our end message
         lines = max(0, self._list.GetLineCount() - 2)
@@ -1380,8 +1377,8 @@ class SearchResultScreen(eclib.ControlBox):
             self._job.Cancel()
 
         self._list.Clear()
-        self._job = eclib.TaskThread(self._list, searchmeth, *args, **kwargs)
-        self._job.start()
+        self._job = eclib.TaskObject(self._list, searchmeth, *args, **kwargs)
+        ed_thread.EdThreadPool().QueueJob(self._job.DoTask)
         self._cancelb.Enable()
 
 #-----------------------------------------------------------------------------#
@@ -1418,7 +1415,7 @@ class SearchResultList(eclib.OutputBuffer):
         """
         if isinstance(value, basestring):
             # Regular search result
-            eclib.OutputBuffer.AppendUpdate(self, value)
+            super(SearchResultList, self).AppendUpdate(value)
         else:
             # Search in a new file has started
             self._files += 1
