@@ -18,8 +18,8 @@ specific options such as commenting code.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 67648 2011-04-29 03:39:27Z CJP $"
-__revision__ = "$Revision: 67648 $"
+__svnid__ = "$Id: ed_stc.py 69065 2011-09-11 19:18:25Z CJP $"
+__revision__ = "$Revision: 69065 $"
 
 #-------------------------------------------------------------------------#
 # Imports
@@ -39,6 +39,7 @@ import ed_msg
 import ed_mdlg
 import ed_txt
 from ed_keyh import KeyHandler, ViKeyHandler
+import eclib
 import ebmlib
 import ed_thread
 
@@ -413,10 +414,12 @@ class EditraStc(ed_basestc.EditraBaseStc):
         @keyword upper: Flag whether conversion is to upper case or not.
 
         """
+        sel = self.GetSelectedText()
         if upper:
-            self.UpperCase()
+            sel = sel.upper()
         else:
-            self.LowerCase()
+            sel = sel.lower()
+        self.ReplaceSelection(sel)
 
     def EnableAutoBackup(self, enable):
         """Enable automatic backups
@@ -1579,6 +1582,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.LOG("[ed_stc][evt] Code Folding Turned On")
             self._config['folding'] = True
             self.SetMarginWidth(ed_basestc.FOLD_MARGIN, 12)
+            self.SetProperty("fold", "1")
         else:
             self.LOG("[ed_stc][evt] Code Folding Turned Off")
             self._config['folding'] = False
@@ -1587,6 +1591,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.ExpandAll()
 
             self.SetMarginWidth(ed_basestc.FOLD_MARGIN, 0)
+            self.SetProperty("fold", "0")
 
     def SyntaxOnOff(self, switch=None):
         """Turn Syntax Highlighting on and off
@@ -1860,10 +1865,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
                         is re-saved if it has an on disk file.
 
         """
-        self.Freeze()
-        while self.CanUndo():
-            self.Undo()
-        self.Thaw()
+        with eclib.Freezer(self) as _tmp:
+            while self.CanUndo():
+                self.Undo()
 
         fname = self.GetFileName()
         if len(fname):
@@ -1871,13 +1875,12 @@ class EditraStc(ed_basestc.EditraBaseStc):
 
     def RevertToSaved(self):
         """Revert the current buffer back to the last save point"""
-        self.Freeze()
-        while self.CanUndo():
-            if self.GetModify():
-                self.Undo()
-            else:
-                break
-        self.Thaw()
+        with eclib.Freezer(self) as _tmp:
+            while self.CanUndo():
+                if self.GetModify():
+                    self.Undo()
+                else:
+                    break
 
     def SaveFile(self, path):
         """Save buffers contents to disk
@@ -1935,10 +1938,12 @@ class EditraStc(ed_basestc.EditraBaseStc):
         """
         super(EditraStc, self).ConfigureLexer(file_ext)
 
+        if not self._config['folding']:
+            self.SetProperty("fold", "0")
+
         # Notify that lexer has changed
-        self.LOG("[ed_stc][info] Lexer change notification for context %d" %
-                 self.GetTopLevelParent().GetId())
+        pid = self.TopLevelParent.Id
+        self.LOG("[ed_stc][info] Lexer change notification for context %d" % pid)
         ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_LEXER,
-                           (self.GetFileName(), self.GetLangId()),
-                           context=self.GetTopLevelParent().GetId())
+                           (self.GetFileName(), self.GetLangId()), pid)
         return True
