@@ -3,7 +3,7 @@
 # Purpose: Editra's style management system. Implements the interpretation of #
 #          Editra Style Sheets to the StyledTextCtrl.                         #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# Copyright: (c) 2008-2011 Cody Precord <staff@editra.org>                    #
 # License: wxWindows License                                                  #
 ###############################################################################
 
@@ -20,8 +20,8 @@ U{http://editra.org/editra_style_sheets}.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_style.py 67609 2011-04-26 19:47:25Z CJP $"
-__revision__ = "$Revision: 67609 $"
+__svnid__ = "$Id: ed_style.py 68615 2011-08-09 19:01:24Z CJP $"
+__revision__ = "$Revision: 68615 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -635,7 +635,13 @@ class StyleMgr(object):
             if reader == -1:
                 self.LOG("[ed_style][err] Failed to open style sheet: %s" % style_sheet)
                 return False
-            ret_val = self.SetStyles(style_sheet, self.ParseStyleData(reader.read()))
+            style_data = None
+            try:
+                style_data = self.ParseStyleData(reader.read())
+            except Exception, msg:
+                self.LOG("[ed_style][err] Failed to parse style data for %s:" % style_sheet)
+                return False
+            ret_val = self.SetStyles(style_sheet, style_data)
             reader.close()
             return ret_val
         elif style_sheet not in StyleMgr.STYLES:
@@ -942,9 +948,8 @@ class StyleMgr(object):
         @postcondition: style scheme is set to specified style
 
         """
-        if spec_style != self.style_set:
+        if spec_style and (spec_style != self.style_set):
             self.LoadStyleSheet(self.GetStyleSheet(spec_style), force=True)
-        self.UpdateBaseStyles()
         self.SetSyntax(self.GetSyntaxParams())
         self.Refresh()
 
@@ -992,9 +997,10 @@ class StyleMgr(object):
             self.SetSelForeground(False, wx.BLACK)
         self.SetSelBackground(True, sback)
 
-        wspace = self.GetItemByName('whitespace_style')
-        self.SetWhitespaceBackground(True, wspace.GetBack())
-        self.SetWhitespaceForeground(True, wspace.GetFore())
+        # Causes issues with selecting text when view whitespace is on
+#        wspace = self.GetItemByName('whitespace_style')
+#        self.SetWhitespaceBackground(True, wspace.GetBack())
+#        self.SetWhitespaceForeground(True, wspace.GetFore())
 
         default_fore = self.GetDefaultForeColour()
         edge_colour = self.GetItemByName('edge_style')
@@ -1069,8 +1075,7 @@ DEF_STYLE_DICT = \
          'stringeol_style' : StyleItem("#000000", "#EEC0EE",
                                        "%(secondary)s", ex=["bold", "eol"]),
          'unknown_style' : StyleItem("#FFFFFF", "#DD0101", ex=["bold", "eol"]),
-         'userkw_style' : StyleItem(),
-         'whitespace_style' : StyleItem('#838383')
+         'userkw_style' : StyleItem()
          }
 
 def MergeFonts(style_dict, font_dict):
