@@ -21,8 +21,8 @@ LANGUAGE: Python
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: synextreg.py 66605 2011-01-06 00:55:52Z CJP $"
-__revision__ = "$Revision: 66605 $"
+__svnid__ = "$Id: synextreg.py 69105 2011-09-16 21:49:07Z CJP $"
+__revision__ = "$Revision: 69105 $"
 
 #-----------------------------------------------------------------------------#
 import os
@@ -407,12 +407,10 @@ class ExtensionRegister(dict):
 
     """
     instance = None
-    first = True
     config = u'synmap'
     def __init__(self):
         """Initializes the register"""
-        if self.first:
-            self.first = False
+        if not ExtensionRegister.instance:
             self.LoadDefault()
 
     def __new__(cls, *args, **kargs):
@@ -437,6 +435,7 @@ class ExtensionRegister(dict):
         and remove associations from older settings.
         @param i: key to set
         @param y: value to set
+        @throws: TypeError Only accepts list() objects
 
         """
         if not isinstance(y, list):
@@ -499,13 +498,15 @@ class ExtensionRegister(dict):
 
     def FileTypeFromExt(self, ext):
         """Returns the file type that is associated with
-        the extension. If no association is found Plain Text
+        the extension. Matching is done with a case insensitive search.
+        If no association is found Plain Text
         will be returned by default.
         @param ext: extension to lookup
 
         """
+        ext = ext.lower()
         for key, val in self.iteritems():
-            if ext in val:
+            if ext in [ext2.lower() for ext2 in val]:
                 return key
         return LANG_TXT
 
