@@ -9,8 +9,8 @@
 """Unittest cases for testing the fileutil functions"""
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: testFileUtil.py 66044 2010-11-05 23:13:22Z CJP $"
-__revision__ = "$Revision: 66044 $"
+__svnid__ = "$Id: testFileUtil.py 69212 2011-09-28 18:47:44Z CJP $"
+__revision__ = "$Revision: 69212 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -43,6 +43,22 @@ class FileUtilTest(unittest.TestCase):
 
     #---- Tests ----#
 
+    def testComparePaths(self):
+        """Test functionality of ComparePaths function"""
+        # Test case sensitivity
+        if ISWINDOWS:
+            self.assertTrue(ebmlib.ComparePaths("C:\\Windows", "C:\\WINDOWS"))
+            self.assertTrue(ebmlib.ComparePaths("C:\\Windows", "C:\\Windows"))
+        else:
+            self.assertFalse(ebmlib.ComparePaths("/usr/bin", "/usr/BIN"))
+            self.assertTrue(ebmlib.ComparePaths("/usr/bin", "/usr/bin"))
+
+        # Test wacky path strings
+        if ISWINDOWS:
+            self.assertTrue(ebmlib.ComparePaths("C:\\Windows\\..\\", "C:\\"))
+        else:
+            self.assertTrue(ebmlib.ComparePaths("/usr/../", "/"))
+
     def testGetAbsPath(self):
         """Test getting a files absolute path"""
         path = os.path.join(u'.', u'data')
@@ -52,7 +68,8 @@ class FileUtilTest(unittest.TestCase):
         if ISWINDOWS:
             path = "c:\\documents and settings"
             spath = "C:\\DOCUME~1"
-            self.assertEquals(path, ebmlib.GetAbsPath(spath).lower())
+            self.assertEquals(path, ebmlib.GetAbsPath(spath).lower(), 
+                              "Missing win32api extension modules?")
 
     def testGetFileExtension(self):
         """Test getting a files extension"""
